<?php

namespace Drupal\action_queue_states;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Routing\RedirectDestinationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\workflows\TransitionInterface;
use Drupal\Core\Url;

/**
 * Defines a class for reacting to entity events.
 *
 * @internal
 */
class EntityOperations implements ContainerInjectionInterface {

  use RedirectDestinationTrait;

  /**
   * The Action Queue Transitions Service.
   *
   * @var \Drupal\action_queue_states\ActionQueueTransitions
   */
  protected $transitions;

  /**
   * Constructs a new EntityOperations object.
   *
   * @param \Drupal\action_queue_states\ActionQueueTransitions $transitions
   *   Action Queue Transitions service.
   */
  public function __construct(ActionQueueTransitions $transitions) {
    $this->transitions = $transitions;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('action_queue_states.transitions'),
    );
  }

  /**
   * Ensures that a destination is present on the given URL.
   *
   * @param \Drupal\Core\Url $url
   *   The URL object to which the destination should be added.
   *
   * @return \Drupal\Core\Url
   *   The updated URL object.
   */
  protected function ensureDestination(Url $url) {
    return $url->mergeOptions(['query' => $this->getRedirectDestination()->getAsArray()]);
  }

  /**
   * Creates the transition link.
   *
   * @param TransitionInterface $transition
   *   The transition.
   * @param EntityInterface $entity
   *   The current entity.
   *
   * @return array
   *   The transition link render array.
   */
  public function createTransitionLink(TransitionInterface $transition, EntityInterface $entity) {
    $options = [
      'action_queue_item' => $entity->id(),
      'transition' => $transition->id(),
    ];

    $url = Url::fromRoute('entity.action_queue_item.status_form', $options);
    return [
      'title' => $transition->label(),
      'url' => $this->ensureDestination($url),
      'weight' => 10,
      'access' => $entity->access('status', NULL, TRUE),
    ];
  }

  /**
   * Create the transition operations links.
   *
   * @param EntityInterface $entity
   *   The current entity.
   * @return array
   *   Array of available transition links.
   */
  public function entityOperations(EntityInterface $entity) {
    $valid = $this->transitions->getValidTransitions($entity, \Drupal::currentUser());
    $operations = [];
    if ($entity->hasLinkTemplate('status-form')) {
      foreach ($valid as $transition) {
        $status_link = $this->createTransitionLink($transition, $entity);
        $operations['status_' . $transition->id()] = $status_link;
      }
    }

    return $operations;
  }

}
